@@ -1,12 +1,10 @@
 ;;; org-faces.el --- Face definitions for Org-mode.
 
-;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-;;   Free Software Foundation, Inc.
+;; Copyright (C) 2004-2013 Free Software Foundation, Inc.
 
 ;; Author: Carsten Dominik <carsten at orgmode dot org>
 ;; Keywords: outlines, hypermedia, calendar, wp
 ;; Homepage: http://orgmode.org
-;; Version: 7.7
 ;;
 ;; This file is part of GNU Emacs.
 ;;
@@ -46,6 +44,9 @@
 	  (set-face-doc-string new-face docstring)))))
 (put 'org-copy-face 'lisp-indent-function 2)
 
+(when (featurep 'xemacs)
+  (put 'mode-line 'face-alias 'modeline))
+
 (defgroup org-faces nil
   "Faces in Org-mode."
   :tag "Org Faces"
@@ -201,7 +202,7 @@ set the properties in the `org-column' face.  For example, set
 
 Under XEmacs, the rules are simpler, because the XEmacs version of
 column view defines special faces for each outline level.  See the file
-`org-colview-xemacs.el' for details."
+`org-colview-xemacs.el' in Org's contrib/ directory for details."
   :group 'org-faces)
 
 (defface org-column-title
@@ -216,12 +217,6 @@ column view defines special faces for each outline level.  See the file
   "Face for column display of entry properties."
   :group 'org-faces)
 
-(when (fboundp 'set-face-attribute)
-  ;; Make sure that a fixed-width face is used when we have a column table.
-  (set-face-attribute 'org-column nil
-		      :height (face-attribute 'default :height)
-		      :family (face-attribute 'default :family)))
-
 (defface org-agenda-column-dateline
   (org-compatible-face 'org-column
     '((t nil)))
@@ -287,6 +282,18 @@ column view defines special faces for each outline level.  See the file
   "Face for date/time stamps."
   :group 'org-faces)
 
+(defface org-date-selected
+  (org-compatible-face nil
+    '((((class color) (min-colors 16) (background light)) (:foreground "Red1" :inverse-video t))
+      (((class color) (min-colors 16) (background dark))  (:foreground "Pink" :inverse-video t))
+      (((class color) (min-colors 8)  (background light)) (:foreground "red"  :inverse-video t))
+      (((class color) (min-colors 8)  (background dark))  (:foreground "red"  :inverse-video t))
+      (t (:inverse-video t))))
+  "Face for highlighting the calendar day when using `org-read-date'.
+Using a bold face here might cause discrepancies while displaying the
+calendar."
+  :group 'org-faces)
+
 (defface org-sexp-date
   '((((class color) (background light)) (:foreground "Purple"))
     (((class color) (background dark)) (:foreground "Cyan"))
@@ -301,6 +308,11 @@ Note that the variable `org-tag-faces' can be used to overrule this face for
 specific tags."
   :group 'org-faces)
 
+(defface org-list-dt
+  '((t (:bold t)))
+  "Default face for definition terms in lists."
+  :group 'org-faces)
+
 (defface org-todo ; font-lock-warning-face
   (org-compatible-face nil
     '((((class color) (min-colors 16) (background light)) (:foreground "Red1" :bold t))
@@ -353,6 +365,7 @@ keywords will then be interpreted as either foreground or background
 color."
   :group 'org-faces
   :group 'org-todo
+  :version "24.1"
   :type '(repeat
 	  (cons (choice (const todo) (const tag) (const priority))
 		(choice (const :foreground) (const :background)))))
@@ -372,8 +385,16 @@ determines if it is a foreground or a background color."
 	  (cons
 	   (string :tag "Keyword")
 	   (choice :tag "Face   "
-	    (string :tag "Color")
-	    (sexp :tag "Face")))))
+		   (string :tag "Color")
+		   (sexp :tag "Face")))))
+
+(defface org-priority ;; originally copied from font-lock-string-face
+  (org-compatible-face 'font-lock-keyword-face
+    '((((class color) (min-colors 16) (background light)) (:foreground "RosyBrown"))
+      (((class color) (min-colors 16) (background dark)) (:foreground "LightSalmon"))
+      (t (:italic t))))
+  "Face used for priority cookies."
+  :group 'org-faces)
 
 (defcustom org-priority-faces nil
   "Faces for specific Priorities.
@@ -389,8 +410,8 @@ determines if it is a foreground or a background color."
 	  (cons
 	   (character :tag "Priority")
 	   (choice    :tag "Face    "
-	    (string :tag "Color")
-	    (sexp :tag "Face")))))
+		      (string :tag "Color")
+		      (sexp :tag "Face")))))
 
 (defvar org-tags-special-faces-re nil)
 (defun org-set-tag-faces (var value)
@@ -403,7 +424,7 @@ determines if it is a foreground or a background color."
 (defface org-checkbox
   (org-compatible-face 'bold
     '((t (:bold t))))
-  "Face for checkboxes"
+  "Face for checkboxes."
   :group 'org-faces)
 
 
@@ -430,8 +451,8 @@ changes."
 	  (cons
 	   (string :tag "Tag ")
 	   (choice :tag "Face"
-	    (string :tag "Foreground color")
-	    (sexp :tag "Face")))))
+		   (string :tag "Foreground color")
+		   (sexp :tag "Face")))))
 
 (defface org-table ;; originally copied from font-lock-function-name-face
   (org-compatible-face nil
@@ -475,9 +496,9 @@ changes."
   :version "22.1")
 
 (defface org-document-title
-  '((((class color) (background light)) (:foreground "midnight blue" :weight bold :height 1.44))
-    (((class color) (background dark)) (:foreground "pale turquoise" :weight bold :height 1.44))
-    (t (:weight bold :height 1.44)))
+  '((((class color) (background light)) (:foreground "midnight blue" :weight bold))
+    (((class color) (background dark)) (:foreground "pale turquoise" :weight bold))
+    (t (:weight bold)))
   "Face for document title, i.e. that which follows the #+TITLE: keyword."
   :group 'org-faces)
 
@@ -540,15 +561,16 @@ follows a #+DATE:, #+AUTHOR: or #+EMAIL: keyword."
   :version "22.1")
 
 (org-copy-face 'org-block 'org-quote
-   "Face for #+BEGIN_QUOTE ... #+END_QUOTE blocks.")
+  "Face for #+BEGIN_QUOTE ... #+END_QUOTE blocks.")
 (org-copy-face 'org-block 'org-verse
-   "Face for #+BEGIN_VERSE ... #+END_VERSE blocks.")
+  "Face for #+BEGIN_VERSE ... #+END_VERSE blocks.")
 
 (defcustom org-fontify-quote-and-verse-blocks nil
   "Non-nil means, add a special face to #+begin_quote and #+begin_verse block.
 When nil, format these as normal Org.  This is the default, because the
 content of these blocks will still be treated as Org syntax."
   :group 'org-faces
+  :version "24.1"
   :type 'boolean)
 
 (defface org-clock-overlay ;; copied from secondary-selection
@@ -564,8 +586,8 @@ content of these blocks will still be treated as Org syntax."
       (((class color) (min-colors 8))
        (:background "cyan" :foreground "black"))
       (t (:inverse-video t))))
-    "Basic face for displaying the secondary selection."
-    :group 'org-faces)
+  "Basic face for displaying the secondary selection."
+  :group 'org-faces)
 
 (defface org-agenda-structure ;; originally copied from font-lock-function-name-face
   (org-compatible-face nil
@@ -592,7 +614,7 @@ content of these blocks will still be treated as Org syntax."
   "Face used in agenda for weekend days.
 See the variable `org-agenda-weekend-days' for a definition of which days
 belong to the weekend."
-	       :weight 'bold)
+  :weight 'bold)
 
 (defface org-scheduled
   (org-compatible-face nil
@@ -665,15 +687,30 @@ month and 365.24 days for a year)."
 
 (defface org-agenda-restriction-lock
   (org-compatible-face nil
-    '((((class color) (min-colors 88) (background light)) (:background "yellow1"))
-      (((class color) (min-colors 88) (background dark))  (:background "skyblue4"))
-      (((class color) (min-colors 16) (background light)) (:background "yellow1"))
-      (((class color) (min-colors 16) (background dark))  (:background "skyblue4"))
+    '((((class color) (min-colors 88) (background light)) (:background "#eeeeee"))
+      (((class color) (min-colors 88) (background dark))  (:background "#1C1C1C"))
+      (((class color) (min-colors 16) (background light)) (:background "#eeeeee"))
+      (((class color) (min-colors 16) (background dark))  (:background "#1C1C1C"))
       (((class color) (min-colors 8)) (:background "cyan" :foreground "black"))
       (t (:inverse-video t))))
   "Face for showing the agenda restriction lock."
   :group 'org-faces)
 
+(defface org-agenda-filter-tags
+  (org-compatible-face 'mode-line nil)
+  "Face for tag(s) in the mode-line when filtering the agenda."
+  :group 'org-faces)
+
+(defface org-agenda-filter-regexp
+  (org-compatible-face 'mode-line nil)
+  "Face for regexp(s) in the mode-line when filtering the agenda."
+  :group 'org-faces)
+
+(defface org-agenda-filter-category
+  (org-compatible-face 'mode-line nil)
+  "Face for categories(s) in the mode-line when filtering the agenda."
+  :group 'org-faces)
+
 (defface org-time-grid ;; originally copied from font-lock-variable-name-face
   (org-compatible-face nil
     '((((class color) (min-colors 16) (background light)) (:foreground "DarkGoldenrod"))
@@ -686,15 +723,24 @@ month and 365.24 days for a year)."
   "Face used to show the current time in the time grid.")
 
 (defface org-agenda-diary
-  (org-compatible-face 'default
-    nil)
+  (org-compatible-face 'default nil)
   "Face used for agenda entries that come from the Emacs diary."
   :group 'org-faces)
 
+(defface org-agenda-calendar-event
+  (org-compatible-face 'default nil)
+  "Face used to show events and appointments in the agenda."
+  :group 'org-faces)
+
+(defface org-agenda-calendar-sexp
+  (org-compatible-face 'default nil)
+  "Face used to show events computed from a S-expression."
+  :group 'org-faces)
+
 (defconst org-level-faces
   '(org-level-1 org-level-2 org-level-3 org-level-4
-    org-level-5 org-level-6 org-level-7 org-level-8
-    ))
+		org-level-5 org-level-6 org-level-7 org-level-8
+		))
 
 (defcustom org-n-level-faces (length org-level-faces)
   "The number of different faces to be used for headlines.
@@ -704,15 +750,16 @@ If it is less than 8, the level-1 face gets re-used for level N+1 etc."
   :group 'org-faces)
 
 (defcustom org-cycle-level-faces t
- "Non-nil means level styles cycle after level `org-n-level-faces'.
+  "Non-nil means level styles cycle after level `org-n-level-faces'.
 Then so level org-n-level-faces+1 is styled like level 1.
 If nil, then all levels >=org-n-level-faces are styled like
 level org-n-level-faces"
- :group 'org-appearance
- :group 'org-faces
- :type 'boolean)
+  :group 'org-appearance
+  :group 'org-faces
+  :version "24.1"
+  :type 'boolean)
 
-(defface org-latex-and-export-specials
+(defface org-latex-and-related
   (let ((font (cond ((assq :inherit custom-face-attributes)
 		     '(:inherit underline))
 		    (t '(:underline t)))))
@@ -725,17 +772,31 @@ level org-n-level-faces"
       (((class color) (background dark))
        (:foreground "burlywood"))
       (t (,@font))))
-  "Face used to highlight math latex and other special exporter stuff."
-  :group 'org-faces)
+  "Face used to highlight LaTeX data, entities and sub/superscript."
+  :group 'org-faces
+  :version "24.4"
+  :package-version '(Org . "8.0"))
 
-(org-copy-face 'modeline 'org-mode-line-clock
+(defface org-macro
+  (org-compatible-face 'org-latex-and-related nil)
+  "Face for macros."
+  :group 'org-faces
+  :version "24.4"
+  :package-version '(Org . "8.0"))
+
+(defface org-tag-group
+  (org-compatible-face 'org-tag nil)
+  "Face for group tags."
+  :group 'org-faces
+  :version "24.4"
+  :package-version '(Org . "8.0"))
+
+(org-copy-face 'mode-line 'org-mode-line-clock
   "Face used for clock display in mode line.")
-(org-copy-face 'modeline 'org-mode-line-clock-overrun
+(org-copy-face 'mode-line 'org-mode-line-clock-overrun
   "Face used for clock display for overrun tasks in mode line."
   :background "red")
 
 (provide 'org-faces)
 
-;; arch-tag: 9dab5f91-c4b9-4d6f-bac3-1f6211ad0a04
-
 ;;; org-faces.el ends here
